/* This file is auto-generated from insns.dat by insns.pl - don't edit it */

/* This file in included by names.c */

static const char *insn_names[] = {
	"aaa",
	"aad",
	"aam",
	"aas",
	"adc",
	"add",
	"addpd",
	"addps",
	"addsd",
	"addss",
	"addsubpd",
	"addsubps",
	"and",
	"andnpd",
	"andnps",
	"andpd",
	"andps",
	"arpl",
	"bound",
	"bsf",
	"bsr",
	"bswap",
	"bt",
	"btc",
	"btr",
	"bts",
	"call",
	"cbw",
	"cdq",
	"clc",
	"cld",
	"clflush",
	"cli",
	"clts",
	"cmc",
	"cmp",
	"cmpeqpd",
	"cmpeqps",
	"cmpeqsd",
	"cmpeqss",
	"cmplepd",
	"cmpleps",
	"cmplesd",
	"cmpless",
	"cmpltpd",
	"cmpltps",
	"cmpltsd",
	"cmpltss",
	"cmpneqpd",
	"cmpneqps",
	"cmpneqsd",
	"cmpneqss",
	"cmpnlepd",
	"cmpnleps",
	"cmpnlesd",
	"cmpnless",
	"cmpnltpd",
	"cmpnltps",
	"cmpnltsd",
	"cmpnltss",
	"cmpordpd",
	"cmpordps",
	"cmpordsd",
	"cmpordss",
	"cmppd",
	"cmpps",
	"cmpsb",
	"cmpsd",
	"cmpss",
	"cmpsw",
	"cmpunordpd",
	"cmpunordps",
	"cmpunordsd",
	"cmpunordss",
	"cmpxchg",
	"cmpxchg486",
	"cmpxchg8b",
	"comisd",
	"comiss",
	"cpuid",
	"cvtdq2pd",
	"cvtdq2ps",
	"cvtpd2dq",
	"cvtpd2pi",
	"cvtpd2ps",
	"cvtpi2pd",
	"cvtpi2ps",
	"cvtps2dq",
	"cvtps2pd",
	"cvtps2pi",
	"cvtsd2si",
	"cvtsd2ss",
	"cvtsi2sd",
	"cvtsi2ss",
	"cvtss2sd",
	"cvtss2si",
	"cvttpd2dq",
	"cvttpd2pi",
	"cvttps2dq",
	"cvttps2pi",
	"cvttsd2si",
	"cvttss2si",
	"cwd",
	"cwde",
	"daa",
	"das",
	"db",
	"dd",
	"dec",
	"div",
	"divpd",
	"divps",
	"divsd",
	"divss",
	"dq",
	"dt",
	"dw",
	"emms",
	"enter",
	"equ",
	"f2xm1",
	"fabs",
	"fadd",
	"faddp",
	"fbld",
	"fbstp",
	"fchs",
	"fclex",
	"fcmovb",
	"fcmovbe",
	"fcmove",
	"fcmovnb",
	"fcmovnbe",
	"fcmovne",
	"fcmovnu",
	"fcmovu",
	"fcom",
	"fcomi",
	"fcomip",
	"fcomp",
	"fcompp",
	"fcos",
	"fdecstp",
	"fdisi",
	"fdiv",
	"fdivp",
	"fdivr",
	"fdivrp",
	"femms",
	"feni",
	"ffree",
	"ffreep",
	"fiadd",
	"ficom",
	"ficomp",
	"fidiv",
	"fidivr",
	"fild",
	"fimul",
	"fincstp",
	"finit",
	"fist",
	"fistp",
	"fisttp",
	"fisub",
	"fisubr",
	"fld",
	"fld1",
	"fldcw",
	"fldenv",
	"fldl2e",
	"fldl2t",
	"fldlg2",
	"fldln2",
	"fldpi",
	"fldz",
	"fmul",
	"fmulp",
	"fnclex",
	"fndisi",
	"fneni",
	"fninit",
	"fnop",
	"fnsave",
	"fnstcw",
	"fnstenv",
	"fnstsw",
	"fpatan",
	"fprem",
	"fprem1",
	"fptan",
	"frndint",
	"frstor",
	"fsave",
	"fscale",
	"fsetpm",
	"fsin",
	"fsincos",
	"fsqrt",
	"fst",
	"fstcw",
	"fstenv",
	"fstp",
	"fstsw",
	"fsub",
	"fsubp",
	"fsubr",
	"fsubrp",
	"ftst",
	"fucom",
	"fucomi",
	"fucomip",
	"fucomp",
	"fucompp",
	"fwait",
	"fxam",
	"fxch",
	"fxrstor",
	"fxsave",
	"fxtract",
	"fyl2x",
	"fyl2xp1",
	"haddpd",
	"haddps",
	"hlt",
	"hsubpd",
	"hsubps",
	"ibts",
	"icebp",
	"idiv",
	"imul",
	"in",
	"inc",
	"incbin",
	"insb",
	"insd",
	"insw",
	"int",
	"int01",
	"int03",
	"int1",
	"int3",
	"into",
	"invd",
	"invlpg",
	"iret",
	"iretd",
	"iretw",
	"jcxz",
	"jecxz",
	"jmp",
	"jmpe",
	"lahf",
	"lar",
	"lddqu",
	"ldmxcsr",
	"lds",
	"lea",
	"leave",
	"les",
	"lfence",
	"lfs",
	"lgdt",
	"lgs",
	"lidt",
	"lldt",
	"lmsw",
	"loadall",
	"loadall286",
	"lodsb",
	"lodsd",
	"lodsw",
	"loop",
	"loope",
	"loopne",
	"loopnz",
	"loopz",
	"lsl",
	"lss",
	"ltr",
	"maskmovdqu",
	"maskmovq",
	"maxpd",
	"maxps",
	"maxsd",
	"maxss",
	"mfence",
	"minpd",
	"minps",
	"minsd",
	"minss",
	"monitor",
	"mov",
	"movapd",
	"movaps",
	"movd",
	"movddup",
	"movdq2q",
	"movdqa",
	"movdqu",
	"movhlps",
	"movhpd",
	"movhps",
	"movlhps",
	"movlpd",
	"movlps",
	"movmskpd",
	"movmskps",
	"movntdq",
	"movnti",
	"movntpd",
	"movntps",
	"movntq",
	"movq",
	"movq2dq",
	"movsb",
	"movsd",
	"movshdup",
	"movsldup",
	"movss",
	"movsw",
	"movsx",
	"movupd",
	"movups",
	"movzx",
	"mul",
	"mulpd",
	"mulps",
	"mulsd",
	"mulss",
	"mwait",
	"neg",
	"nop",
	"not",
	"or",
	"orpd",
	"orps",
	"out",
	"outsb",
	"outsd",
	"outsw",
	"packssdw",
	"packsswb",
	"packuswb",
	"paddb",
	"paddd",
	"paddq",
	"paddsb",
	"paddsiw",
	"paddsw",
	"paddusb",
	"paddusw",
	"paddw",
	"pand",
	"pandn",
	"pause",
	"paveb",
	"pavgb",
	"pavgusb",
	"pavgw",
	"pcmpeqb",
	"pcmpeqd",
	"pcmpeqw",
	"pcmpgtb",
	"pcmpgtd",
	"pcmpgtw",
	"pdistib",
	"pextrw",
	"pf2id",
	"pf2iw",
	"pfacc",
	"pfadd",
	"pfcmpeq",
	"pfcmpge",
	"pfcmpgt",
	"pfmax",
	"pfmin",
	"pfmul",
	"pfnacc",
	"pfpnacc",
	"pfrcp",
	"pfrcpit1",
	"pfrcpit2",
	"pfrsqit1",
	"pfrsqrt",
	"pfsub",
	"pfsubr",
	"pi2fd",
	"pi2fw",
	"pinsrw",
	"pmachriw",
	"pmaddwd",
	"pmagw",
	"pmaxsw",
	"pmaxub",
	"pminsw",
	"pminub",
	"pmovmskb",
	"pmulhriw",
	"pmulhrwa",
	"pmulhrwc",
	"pmulhuw",
	"pmulhw",
	"pmullw",
	"pmuludq",
	"pmvgezb",
	"pmvlzb",
	"pmvnzb",
	"pmvzb",
	"pop",
	"popa",
	"popad",
	"popaw",
	"popf",
	"popfd",
	"popfw",
	"por",
	"prefetch",
	"prefetchnta",
	"prefetcht0",
	"prefetcht1",
	"prefetcht2",
	"prefetchw",
	"psadbw",
	"pshufd",
	"pshufhw",
	"pshuflw",
	"pshufw",
	"pslld",
	"pslldq",
	"psllq",
	"psllw",
	"psrad",
	"psraw",
	"psrld",
	"psrldq",
	"psrlq",
	"psrlw",
	"psubb",
	"psubd",
	"psubq",
	"psubsb",
	"psubsiw",
	"psubsw",
	"psubusb",
	"psubusw",
	"psubw",
	"pswapd",
	"punpckhbw",
	"punpckhdq",
	"punpckhqdq",
	"punpckhwd",
	"punpcklbw",
	"punpckldq",
	"punpcklqdq",
	"punpcklwd",
	"push",
	"pusha",
	"pushad",
	"pushaw",
	"pushf",
	"pushfd",
	"pushfw",
	"pxor",
	"rcl",
	"rcpps",
	"rcpss",
	"rcr",
	"rdmsr",
	"rdpmc",
	"rdshr",
	"rdtsc",
	"resb",
	"resd",
	"resq",
	"rest",
	"resw",
	"ret",
	"retf",
	"retn",
	"rol",
	"ror",
	"rsdc",
	"rsldt",
	"rsm",
	"rsqrtps",
	"rsqrtss",
	"rsts",
	"sahf",
	"sal",
	"salc",
	"sar",
	"sbb",
	"scasb",
	"scasd",
	"scasw",
	"sfence",
	"sgdt",
	"shl",
	"shld",
	"shr",
	"shrd",
	"shufpd",
	"shufps",
	"sidt",
	"sldt",
	"smi",
	"smint",
	"smintold",
	"smsw",
	"sqrtpd",
	"sqrtps",
	"sqrtsd",
	"sqrtss",
	"stc",
	"std",
	"sti",
	"stmxcsr",
	"stosb",
	"stosd",
	"stosw",
	"str",
	"sub",
	"subpd",
	"subps",
	"subsd",
	"subss",
	"svdc",
	"svldt",
	"svts",
	"syscall",
	"sysenter",
	"sysexit",
	"sysret",
	"test",
	"ucomisd",
	"ucomiss",
	"ud0",
	"ud1",
	"ud2",
	"umov",
	"unpckhpd",
	"unpckhps",
	"unpcklpd",
	"unpcklps",
	"verr",
	"verw",
	"wait",
	"wbinvd",
	"wrmsr",
	"wrshr",
	"xadd",
	"xbts",
	"xchg",
	"xlat",
	"xlatb",
	"xor",
	"xorpd",
	"xorps",
	"xstore"
};

/* Conditional instructions */
static const char *icn[] = {
	"cmov",
	"j",
	"set"
};

/* and the corresponding opcodes */
static int ico[] = {
	I_CMOVcc,
	I_Jcc,
	I_SETcc
};
